From 6602dcdd06753d21006b1b33c0154af6c751b98a Mon Sep 17 00:00:00 2001
From: Francesco Chemolli <li@kinkie.it>
Date: Mon, 24 Nov 1997 22:33:28 +0100
Subject: [PATCH] Updated to msqlmod-1.1.

Rev: lib/modules/Sql.pmod/msql.pike:1.2
Rev: src/modules/Msql/ChangeLog:1.2
Rev: src/modules/Msql/configure.in:1.2
Rev: src/modules/Msql/doc/msql:1.2
Rev: src/modules/Msql/msql_config.h.in:1.2
Rev: src/modules/Msql/msqlmod.c:1.4
Rev: src/modules/Msql/version.h:1.1
---
 .gitattributes                    |   1 +
 lib/modules/Sql.pmod/msql.pike    |  10 ++
 src/modules/Msql/ChangeLog        |  50 ++++++++++
 src/modules/Msql/configure.in     |   2 +-
 src/modules/Msql/doc/msql         | 148 ++++++++++++++++++++----------
 src/modules/Msql/msql_config.h.in |   2 +-
 src/modules/Msql/msqlmod.c        |  41 ++++++---
 src/modules/Msql/version.h        |   8 ++
 8 files changed, 202 insertions(+), 60 deletions(-)
 create mode 100644 src/modules/Msql/version.h

diff --git a/.gitattributes b/.gitattributes
index f809146775..ea1a5f398d 100644
--- a/.gitattributes
+++ b/.gitattributes
@@ -101,6 +101,7 @@ testfont binary
 /src/modules/Msql/configure.in foreign_ident
 /src/modules/Msql/msql_config.h.in foreign_ident
 /src/modules/Msql/msqlmod.c foreign_ident
+/src/modules/Msql/version.h foreign_ident
 /src/modules/Mysql/Makefile.in foreign_ident
 /src/modules/Mysql/acconfig.h foreign_ident
 /src/modules/Mysql/configure.in foreign_ident
diff --git a/lib/modules/Sql.pmod/msql.pike b/lib/modules/Sql.pmod/msql.pike
index ef56f553bc..9d09ba16c2 100644
--- a/lib/modules/Sql.pmod/msql.pike
+++ b/lib/modules/Sql.pmod/msql.pike
@@ -1,3 +1,13 @@
 // Msql module support stuff, (C) 1997 Francesco Chemolli <kinkie@comedia.it>
 
 inherit Msql.msql;
+
+//in c it's too hard, we're better off doing it in pike
+mapping(string:mapping(string:mixed)) list_fields(string table, string|void wild) {
+	mapping result = ::list_fields(table);
+	if (!wild||!strlen(wild))
+		return result;
+	array a=glob(wild,indices(result));
+	return mkmapping(a,Array.map(a,lambda(string s, mapping m) 
+			{return m[s];},result));
+}
diff --git a/src/modules/Msql/ChangeLog b/src/modules/Msql/ChangeLog
index ee57e1a2d0..a3812e0264 100644
--- a/src/modules/Msql/ChangeLog
+++ b/src/modules/Msql/ChangeLog
@@ -1,9 +1,57 @@
+Fri Nov 21 14:58:45 1997  Francesco Chemolli  <kinkie@ai-chan>
+
+	* version.h: Released version 1.1
+
+	* lib/msql.pike:
+	Finally the driver is completely interface-compliant. This class puts
+	a small wrapper around the list_fields function, which before was
+	not completely compliant.
+
+	* doc/msql:
+	Updated and corrected the documentation, in preparation of releasing
+	version 1.1 of the driver.
+
+	* doc/msql: Changes to the documentation to better reflect Pike/0.5.
+
+Thu Nov 13 10:46:33 1997  Francesco Chemolli  <kinkie@ai-chan>
+
+	* ChangeLog: *** empty log message ***
+
+	* version.h: Released version 1.1pre7.
+
+	* msqlmod.c: Ironed out some compile-time warnings.
+
+	* msqlmod.c: Added driver versioning.
+
+Thu Nov 13 10:44:38 1997  Francesco Chemolli  <kinkie@ai-chan>
+
+	* version.h: Released version 1.1pre7.
+
+	* msqlmod.c: Ironed out some compile-time warnings.
+
+	* msqlmod.c: Added driver versioning.
+
+Wed Aug 27 11:33:47 1997  Francesco Chemolli  <kinkie@ai-chan>
+
+	* ChangeLog, msqlmod.c:
+	Added support for the new hostname parameter specification.
+	Now it accepts "hostname:port"-style arguments.
+	However, due to then mSQL architecture, the port is silently ignored.
+
+Mon Jun 30 13:57:46 1997  Francesco Chemolli  <kinkie@ai-chan>
+
+	* configure.in, configure:
+	Configure fixes which allow much more flexibility in configuration and
+	maintainance.
+
 Mon Jun 30 13:57:46 1997  Francesco Chemolli  <kinkie@ai-chan>
 
 	* configure.in, configure:
 	Configure fixes which allow much more flexibility in configuration and
 	maintainance.
 
+	* released version 1.1pre5
+
 Sun Jun 29 20:05:50 1997  Francesco Chemolli  <kinkie@ai-chan>
 
 	* msqlmod.c:
@@ -36,6 +84,8 @@ Fri Jun 13 21:42:48 1997  Francesco Chemolli  <kinkie@ai-chan>
 
 	* configure, dependencies: Aggiunto il file vuoto "dependencies".
 
+	* released version 1.1pre4
+
 Wed Apr  2 11:31:44 1997  Francesco Chemolli  <kinkie@ai-chan>
 
 	* COPYING, ChangeLog, INSTALL, Makefile.in, NOTES, configure, configure.in, doc/msql, msqlmod.c, test_msqlmod.pike, testsuite.in:
diff --git a/src/modules/Msql/configure.in b/src/modules/Msql/configure.in
index 0bd2e56df1..f0231f2300 100644
--- a/src/modules/Msql/configure.in
+++ b/src/modules/Msql/configure.in
@@ -1,4 +1,4 @@
-dnl $Id: configure.in,v 1.1.1.1 1997/10/14 21:55:42 grubba Exp $
+dnl $Id: configure.in,v 1.2 1997/11/24 21:32:14 grubba Exp $
 dnl (C) 1997 Francesco Chemolli <kinkie@comedia.it>
 
 AC_INIT(msqlmod.c)
diff --git a/src/modules/Msql/doc/msql b/src/modules/Msql/doc/msql
index bb7d9182bf..9e6f3ef954 100644
--- a/src/modules/Msql/doc/msql
+++ b/src/modules/Msql/doc/msql
@@ -1,5 +1,5 @@
 NAME
-	/precompiled/sql/msql - mSQL SQL server interface
+	Sql.msql - mSQL SQL server interface
 
 AUTHOR
 	Francesco Chemolli <kinkie@comedia.it>
@@ -13,31 +13,41 @@ DESCRIPTION
 	you can connect to the database over a TCP/IP socket
 
 	Please notice that unless you wish to specifically connect to a mSQL
-	server, you'd better use the /precompiled/sql program instead.
-	I and Henrik Grubbstrom have (hopefully) designed a server-independent
+	server, you'd better use the Sql.sql program instead.
+	I and Henrik Grubbström have (hopefully) designed a server-independent
 	sql-server-class. The interfaces to all existing sql-classes
-	are consistant. Using /precompiled/sql ensures that your pike
+	are consistent. Using Sql.sql ensures that your pike
 	applications will run with any supported SQL server without changing
 	a single line of code.
 
-	Also notice that we are currently undergoing a major release change
-	for mSQL. Some functions may mSQL/2.0 -specific, and thus missing on
-	hosts runnign mSQL/1.0.*
+	Also notice that some functions may mSQL/2.0 -specific, and thus missing on
+	hosts running mSQL/1.0.*
 
 SEE ALSO
-	/precompiled/sql/mysql, /precompiled/sql
+	Sql.sql
 
 ============================================================================
 NAME
 	create - initialize module and/or select database
 
 SYNTAX
+	#include <sql.h>
+	object msql();
+	or
+	object Sql.msql();
+	or
+
 	#include <msql.h>
-	void msql->create();
+	object msql(); //deprecated
+	or
+	object Msql.msql(); //deprecated
+
+	Arguments (apply to all forementioned variants):
+	msql->create();
 	or
-	void msql->create (string dbserver);
+	msql->create(string hostname);
 	or
-	void msql->create (string dbserver, string dbname);
+	msql->create(string hostname, string dbname);
 
 DESCRIPTION
 	With one argument, this function
@@ -49,12 +59,16 @@ DESCRIPTION
 	With no arguments it tries to connect to the server on localhost, using
 	UNIX sockets.
 
+	The Msql.msql() syntax is deprecated. Please use the Sql.msql() program
+	instead.
+
 NOTA BENE
-	You need to have a database selected before using it, otherwise you'll
-	get exceptions when you try to query it.
+	You need to have a database selected before using the sql-object, 
+	otherwise you'll get exceptions when you try to query it.
 	Also notice that this function CAN raise exceptions if the db
 	server doesn't respond, if the database doesn't exist or is not accessible
 	by you.
+
 	You don't need bothering about syncronizing the connection to the database:
 	it is automatically closed (and the database is sync-ed) when the msql
 	object is destroyed.
@@ -73,16 +87,16 @@ SYNTAX
 DESCRIPTION
 	Before querying a database you have to select it. This can be accomplished
 	in two ways: the first is calling the create() function with two arguments,
-	another is calling it with only one argument and then calling select_db().
+	another is calling it with one or no argument and then calling select_db().
 	You can also use this function to change the database you're querying,
-	as long as it is on the same server you connected earlier.
+	as long as it is on the same server you are connected to.
 
 NOTA BENE
 	This function CAN raise exceptions in case something goes wrong
 	(for example: unexistant database, insufficient permissions, whatever).
 
 SEE ALSO
-	msql->create
+	msql->create, msql->error
 
 ============================================================================
 NAME
@@ -100,15 +114,18 @@ DESCRIPTION
 	The structure is an array (one entry for each returned row) of mappings
 	which have the column name as index and the column contents as data.
 	So to access a result from the first example you would have to do
-	"results[0]->foo"
+	"results[0]->foo".
+
 	A query which returns no data results in an empty array (and NOT in a 0).
 	Also notice that when there is a column name clash (that is: when you
 	join two or more tables which have columns with the same name), the
 	clashing columns are renamed to <tablename>+"."+<column name>.
 	To access those you'll have to use the indexing operator '[]
 	(i.e.: results[0]["foo.bar"]).
+
 	Errors (both from the interface and the SQL server) are reported via
-	exceptions, you may definitely want to catch them.
+	exceptions, and you definitely want to catch them.
+
 	Also note that if the query is NOT a of SELECT type, but UPDATE or
 	MODIFY, the returned value is an empty array.
 	THIS IS NOT AN ERROR. Errors are reported ONLY via exceptions.
@@ -127,13 +144,13 @@ SYNTAX
 	#include <msql.h>
 	array(string) list_dbs();
 	or
-	array(string) list_dbs(string wildcard);
+	array(string) list_dbs(string glob);
 
 DESCRIPTION
 	Returns an array containing the names of all databases availible on
 	the system. Will throw an exception if there is no server connected.
 	If an argument is specified, it will return only those databases
-	whose name matches the given wildcard (which must be a glob).
+	whose name matches the given glob.
 
 ============================================================================
 NAME
@@ -143,49 +160,47 @@ SYNTAX
 	#include <msql.h>
 	array(string) list_tables();
 	or
-	array(string) list_tables(string wildcard);
+	array(string) list_tables(string glob);
 
 DESCRIPTION
 	Returns an array containing the names of all the tables in the currently
 	selected database. Will throw an exception if we aren't connected to
 	a database.
 	If an argument is specified, it will return only those tables
-	whose name matches the given wildcard (which must be a glob).
+	whose name matches the given glob.
 
 ============================================================================
 NAME
 	list_fields - describe the fields of a table in the database.
 
 SYNTAX
-	mapping(string:mapping(string:mixed)) list_fields(string table);
+	mapping(string:mapping(string:mixed)) list_fields(string table, void|string glob);
 
 DESCRIPTION
 	Returns a mapping describing the fields of a table in the database.
 	The returned value is a mapping, indexed on the column name,
 	of mappings.
-	These contain curretly the fields:
+	The glob argument, if present, filters out the fields not matching the glob.
+	These contain currently the fields:
 
-		"type"(string) describes the field's mSQL data type ("char","integer",...)
+		string "type" describes the field's mSQL data type ("char","integer",...)
 
-		"length"(int) it describes the field's length. It is only interesting for
-		char fields, in fact. Notice that mSQL/2.0's "text" fields
-		make this type obsolete, and this information useless. Also
+		int "length" it describes the field's length. It is only interesting for
+		char() fields, in fact.  Also
 		notice that the strings returned by msql->query() have the correct length.
 		This field only specifies the _maximum_ length a "char()" field can have.
 
-		"table"(string) The table this field is in. Added only for
+		string "table" The table this field is in. Added only for
 		interface compliancy.
 
-		"flags"(multiset(string)) it's a multiset containing textual
+		(multiset(string)) "flags" it's a multiset containing textual
 		descriptions of the server's flags associated with the current field.
 		Currently it can be empty, or contain "unique" or "not null".
 
-
-
 WARNING
-	As of version 1.0 of the mSQL module and 1.4 of the mysql module, this
-	function is _NOT_ generic-sql-interface compliant (it misses the
-	second argument, and this will break the interface).
+	The version of this function in the Msql.msql() program is _NOT_ 
+	sql-interface compliant (this is the main reason why using that program
+	directly is deprecated).
 
 SEE ALSO
 	msql->query
@@ -199,7 +214,9 @@ SYNTAX
 
 DESCRIPTION
 	This function returns the textual description of the last server-related
-	error. Returns 0 if no error has occurred yet.
+	error. Returns 0 if no error has occurred yet. It is not cleared upon
+	reading (can be invoked multiple times, will return the same result
+	until a new error occurs).
 
 SEE ALSO
 	msql->query
@@ -218,7 +235,7 @@ DESCRIPTION
 	module.
 
 SEE ALSO
-	/precompiled/sql
+	Sql.sql
 
 ============================================================================
 NAME
@@ -242,6 +259,9 @@ DESCRIPTION
 	This function creates a new database with the given name (assuming we
 	have enough permissions to do this).
 
+SEE ALSO
+	msql->drop_db
+
 ============================================================================
 NAME
 	drop_db - destroy a database
@@ -253,6 +273,9 @@ DESCRIPTION
 	This function destroys a database and all the data it contains (assuming
 	we have enough permissions to do so). USE WITH CAUTION!
 
+SEE ALSO
+	msql->create_db
+
 ============================================================================
 NAME
 	shutdown - shut down a server
@@ -273,6 +296,14 @@ SYNTAX
 DESCRIPTION
 	This function forces a server to reload its ACLs.
 
+NOTA BENE
+	This function is _NOT_ part of the standard interface, so it is NOT
+	availible through the Sql.sql interface, but only through Sql.msql and
+	Msql.msql programs
+
+SEE ALSO
+	msql->create
+
 ============================================================================
 NAME
 	affected_rows - show how many rows were 'touched' by last operation
@@ -285,10 +316,17 @@ DESCRIPTION
 	our last SQL query.
 
 NOTA BENE
-	This function is availible only if we're supporting mSQL server version 2
+	This function is availible only if you're using mSQL version 2
 	or later. (That means: if the includes and library of version 2 of mSQL
 	were availible when the module was compiled).
 
+	This function is _NOT_ part of the standard interface, so it is NOT
+	availible through the Sql.sql interface, but only through Sql.msql and
+	Msql.msql programs
+
+SEE ALSO
+	msql->create
+
 ============================================================================
 NAME
 	list_index - show the index structure for a table
@@ -305,9 +343,27 @@ DESCRIPTION
 	the mSQL API).
 
 NOTA BENE
-	This function is availible if support the module is for mSQL version 2
+	This function is availible if you're using mSQL version 2
 	or later.
 
+	This function is _NOT_ part of the standard interface, so it is NOT
+	availible through the Sql.sql interface, but only through Sql.msql and
+	Msql.msql programs
+
+SEE ALSO
+	msql->create
+
+============================================================================
+NAME
+	version - show the version of the mSQL driver.
+
+SYNTAX
+	string Msql.version;
+
+DESCRIPTION
+	Should you need to report a bug to the author, please submit along with
+	the report the driver version number, as returned by this call.
+
 ============================================================================
 EXTERNAL INFLUENCES on the module.
 	These ones do not depend on my implementation really, they're just part
@@ -322,23 +378,21 @@ EXTERNAL INFLUENCES on the module.
 	for further details.
 
 NOTES, TODOs AND OTHER THINGS
-	This program is my first attempt at touching the Pike source code, so
-	I may well have messed everything up. I hope I didn't.
-
 	Note that although the database engine should theoretically
 	store any kind of values, the interface seems to be quite text-oriented:
 	strings are returned as null-terminated char*, and not as a
-	(datum,len) couple. For now I followed this trend. I will change this
+	(datum,lenght) couple. For now I followed this trend. I will change this
 	behavior as soon as I can think of a safe way.
 
 	I didn't make any testsuite for this module, since to test anything it would
 	require a working mSQL server, and I can't make any assumption
 	on those... You can try to use the included "test_msql.pike" script,
-	but you'll possibly have to patch it.
+	but you'll probably have to patch it to reflect your site's settings.
 
 	Also note that THIS MODULE USES BLOCKING I/O to connect to the server.
 	mSQL should be reasonably fast, but you might want to consider this
-	particular aspect. It should however be thread-safe.
+	particular aspect. It is thread-safe, and so it can be used
+	in a multithread environment.
 
 THANKS
 	Many thanks to Henrik Grubbström, without whose help this piece of
@@ -362,10 +416,10 @@ COPYRIGHT
 	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 	The mSQL (Mini-SQL) server and its accompanying tools and documentation are 
-	copyright 1993-1996 Hughes Technologies Pty Ltd, and are free to use
+	copyright 1993-1997 Hughes Technologies Pty Ltd, and are free to use
 	for non-commercial entities. For informations about Mini SQL please
 	consult the Hughes Thechnologies WWW site, at http://hughes.com.au/
 
 	The Pike programming language is copyright Frederik Hubinette, and is
 	distributed under the terms of the GNU General Public License. For furhter
-	information, please consult the Pike WWW site, at http://pike.infovav.se/
+	information, please consult the Pike WWW site, at http://pike.idonex.se/
diff --git a/src/modules/Msql/msql_config.h.in b/src/modules/Msql/msql_config.h.in
index bb1bf04953..06ca90de97 100644
--- a/src/modules/Msql/msql_config.h.in
+++ b/src/modules/Msql/msql_config.h.in
@@ -1,4 +1,4 @@
-/* $Id: msql_config.h.in,v 1.1.1.1 1997/10/14 21:55:42 grubba Exp $ */
+/* $Id: msql_config.h.in,v 1.2 1997/11/24 21:32:15 grubba Exp $ */
 #undef STDC_HEADERS
 
 #undef HAVE_MSQL_H
diff --git a/src/modules/Msql/version.h b/src/modules/Msql/version.h
new file mode 100644
index 0000000000..c39db6bd3f
--- /dev/null
+++ b/src/modules/Msql/version.h
@@ -0,0 +1,8 @@
+/*
+ * $Id: version.h,v 1.1 1997/11/24 21:32:16 grubba Exp $
+ *
+ * This file is provided for CVS's sake, and to define a simple string to be
+ * used as the module's release version
+ */
+
+#define MSQLMOD_VERSION "mSQL/1.1"
diff --git a/src/modules/Msql/msqlmod.c b/src/modules/Msql/msqlmod.c
index 1d2de7511f..34ecd02e42 100644
--- a/src/modules/Msql/msqlmod.c
+++ b/src/modules/Msql/msqlmod.c
@@ -2,9 +2,9 @@
  * This code is (C) Francesco Chemolli, 1997.
  * You may use, modify and redistribute it freely under the terms
  * of the GNU General Public License, version 2.
- * $Id: msqlmod.c,v 1.3 1997/11/02 18:34:57 grubba Exp $
+ * $Id: msqlmod.c,v 1.4 1997/11/24 21:32:15 grubba Exp $
  *
  * This versione is intended for Pike/0.5 and later.
  * It won't compile under older versions of the Pike interpreter.
  */
 
@@ -21,21 +21,22 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <machine.h>
-#include <svalue.h>
-#include <threads.h>
 #include <global.h>
+#include <threads.h>
+#include <machine.h>
 #include <interpret.h>
+#include <builtin_functions.h>
+#include <module_support.h>
+#include <svalue.h>
 #include <program.h>
 #include <array.h>
 #include <mapping.h>
 #include <stralloc.h>
-#include <builtin_functions.h>
 #include "operators.h"
 #include "multiset.h"
-#include <module_support.h>
 
-RCSID("$Id: msqlmod.c,v 1.3 1997/11/02 18:34:57 grubba Exp $");
+RCSID("$Id: msqlmod.c,v 1.4 1997/11/24 21:32:15 grubba Exp $");
+#include "version.h"
 
 #ifdef _REENTRANT
 MUTEX_T pike_msql_mutex;
@@ -186,6 +187,7 @@ static void msql_mod_create (INT32 args)
 {
 	struct pike_string * arg1=NULL, *arg2=NULL;
 	int sock, status;
+	char *colon;
 
 	check_all_args("Msql->create",args,
 			BIT_STRING|BIT_VOID,BIT_STRING|BIT_VOID,BIT_STRING|BIT_VOID,
@@ -210,6 +212,17 @@ static void msql_mod_create (INT32 args)
 	}
 
 	THREADS_ALLOW();
+	/* msql won' support specifying a port number to connect to. 
+	 * As far as I know, ':' is not a legal character in an hostname,
+	 * so we'll silently ignore it.
+	 */
+
+	if (arg1) {
+		colon=strchr(arg1->str,':');
+		if (colon)
+			*colon='\0';
+	}
+
 	MSQL_LOCK();
 	/* Warning! If there were no args, we're deferencing a NULL pointer!*/
 	if (!arg1 || !strcmp (arg1->str,"localhost"))
@@ -701,11 +714,14 @@ void pike_module_init(void)
 	 * Third and fourth argument are currently ignored, since mSQL doesn't
 	 * support user/passwd authorization. The user will be the owner of
 	 * the current process.
+	 * The first argument can have the format "hostname:port". Since mSQL
+	 * doesn't support nonstandard ports, that portion is silently ignored,
+	 * and is provided only for generic-interface compliancy
 	 */
 
 	add_function("select_db",select_db,"function(string:void)",
 		OPT_EXTERNAL_DEPEND);
-	/* if no db selected by connect, does it now.
+	/* if no db was selected by connect, does it now.
 	 * CAN raise an exception if there's no such database or we haven't selected
 	 * an host.
 	 */
@@ -767,15 +783,18 @@ void pike_module_init(void)
 	add_function ("affected_rows", do_affected_rows, "function(void:int)",
 		OPT_RETURN|OPT_EXTERNAL_DEPEND);
 	/* Returns the number of rows 'touched' by last query */
-	/* UNTESTED */
 
 	add_function ("list_index", do_list_index, "function(string,string:array)",
 			OPT_EXTERNAL_DEPEND);
 	/* Returns the index structure on the specified table */
-	/* UNTESTED */
 #endif
 
 	end_class("msql",0);
+
+	/* Versioning information, to be obtained as "Msql.version". Mainly a
+	 * convenience for RCS
+	 */
+	add_string_constant("version",MSQLMOD_VERSION,0);
 }
 
 #else /*HAVE_MSQL*/
@@ -2,9 +2,9 @@
  *
- * This versione is intended for Pike/0.5 and later.
+ * This version is intended for Pike/0.5 and later.
  * It won't compile under older versions of the Pike interpreter.
  */
-- 
GitLab